"use strict";
/**
 * Copyright (c) Microsoft Corporation.
 * Licensed under the MIT License.
 *
 * @format
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Dump a section of the native visual tree.
 */
async function dumpVisualTree(accessibilityId, opts) {
    if (!automationClient) {
        throw new Error('RPC client is not enabled');
    }
    const dumpResponse = await automationClient.invoke('DumpVisualTree', {
        accessibilityId,
        ...opts,
    });
    if (dumpResponse.type === 'error') {
        throw new Error(dumpResponse.message);
    }
    const element = dumpResponse.result;
    if ('XamlType' in element && (opts === null || opts === void 0 ? void 0 : opts.pruneCollapsed) !== false) {
        pruneCollapsedElements(element);
    }
    if ('XamlType' in element && (opts === null || opts === void 0 ? void 0 : opts.deterministicOnly) !== false) {
        removeNonDeterministicProps(element);
    }
    if ('XamlType' in element && (opts === null || opts === void 0 ? void 0 : opts.removeDefaultProps) !== false) {
        removeDefaultProps(element);
    }
    if (!('XamlType' in element) && (opts === null || opts === void 0 ? void 0 : opts.removeGuidsFromImageSources) !== false) {
        removeGuidsFromImageSources(element);
    }
    return element;
}
exports.default = dumpVisualTree;
function removeGuidsFromImageSourcesHelper(node) {
    if (node._Props && node._Props.Sources) {
        node._Props.Sources.forEach((source) => {
            if (source.Uri) {
                if (source.Uri.startsWith('blob:')) {
                    source.Uri = source.Uri.replace(/blob:[a-f0-9]+-[a-f0-9]+-[a-f0-9]+-[a-f0-9]+-[a-f0-9]+/, 'blob:<some_guid_here>');
                    source.Uri = source.Uri.replace(/size=\d{5}/, 'size=<size>');
                }
                else if (source.Uri.startsWith('https://www.facebook.com/assets/fb_lite_messaging/E2EE-settings@3x.png?r=1&t=')) {
                    source.Uri =
                        'https://www.facebook.com/assets/fb_lite_messaging/E2EE-settings@3x.png?r=1&t=<some_hash_here>';
                }
                else if (source.Uri.startsWith('https://www.facebook.com/ar_effect/external_textures/648609739826677.png?hash=')) {
                    source.Uri =
                        'https://www.facebook.com/ar_effect/external_textures/648609739826677.png?hash=<some_hash_here>';
                }
                else if (source.Uri.startsWith('https://www.facebook.com/ads/pics/successstories.png?hash=')) {
                    source.Uri =
                        'https://www.facebook.com/ads/pics/successstories.png?hash=<some_hash_here>';
                }
                else {
                    // When getting files from a prebuilt bundle the uri is going to include a local path, which would make snapshots inconsistent,
                    // This logic replaces the local path so that we get consistent results.
                    // file://E:\\repos\\react-native-windows\\packages\\e2e-test-app-fabric\\windows\\RNTesterApp-Fabric.Package\\bin\\x64\\Release\\AppX\\RNTesterApp-Fabric\\Bundle\\@react-native-windows/tester/js/assets/uie_thumb_normal@2x.png
                    // becomes
                    // <localOrBundlerUri>@react-native-windows/tester/js/assets/uie_thumb_normal@2x.png
                    const packagesPath = require('path')
                        .resolve(__dirname, '../../..')
                        .replace(/\\/g, '\\\\');
                    source.Uri = source.Uri.replace(new RegExp(`file://${packagesPath}.*\\\\Bundle\\\\assets/_+`), '<localOrBundlerUri>');
                    // When loading the bundle from metro local paths will be replaced with paths to localhost, which will not align with snapshots made with prebuilt bundles.
                    // This logic replaces the localhost uri, with the same uri that we would have gotten from a prebuild bundle.  This makes it easier to debug without breaking snapshots
                    // http://localhost:8081/assets/@@/@react-native-windows/tester/js/assets/uie_thumb_normal@2x.png?platform=windows&hash=c6f5aec4d9e0aa47c0887e4266796224
                    // becomes
                    // "<localOrBundlerUri>@react-native-windows/tester/js/assets/uie_thumb_normal@2x.png"
                    source.Uri = source.Uri.replace(/http:\/\/localhost:8081\/assets\/(@@\/)+/, '<localOrBundlerUri>');
                    source.Uri = source.Uri.replace(/\?platform=windows&hash=[^=]$/, '');
                }
            }
        });
    }
    if (node.__Children) {
        node.__Children.forEach((child) => removeGuidsFromImageSourcesHelper(child));
    }
}
function removeGuidsFromImageSources(visualTree) {
    removeGuidsFromImageSourcesHelper(visualTree['Component Tree']);
}
/**
 * Removes trees of XAML that are not visible.
 */
function pruneCollapsedElements(element) {
    if (!element.children) {
        return;
    }
    element.children = element.children.filter(child => child.Visibility !== 'Collapsed');
    element.children.forEach(pruneCollapsedElements);
}
/**
 * Removes trees of properties that are not deterministic
 */
function removeNonDeterministicProps(element) {
    if (element.RenderSize) {
        // RenderSize is subject to rounding, etc and should mostly be derived from
        // other deterministic properties in the tree.
        delete element.RenderSize;
    }
    if (element.children) {
        element.children.forEach(removeNonDeterministicProps);
    }
}
/**
 * Removes noise from snapshot by removing properties with the default value
 */
function removeDefaultProps(element) {
    const defaultValues = [['Tooltip', null]];
    defaultValues.forEach(([propname, defaultValue]) => {
        if (element[propname] === defaultValue) {
            delete element[propname];
        }
    });
    if (element.children) {
        element.children.forEach(removeDefaultProps);
    }
}
//# sourceMappingURL=dumpVisualTree.js.map